@@ -132,14 +132,27 @@ struct slot *slot_list = NULL;
 /*
  * length of voice and common MIDI messages (status byte included)
  */
-unsigned int voice_len[] = { 3, 3, 3, 3, 2, 2, 3 };
-unsigned int common_len[] = { 0, 2, 3, 2, 0, 0, 1, 1 };
+const unsigned int voice_len[] = { 3, 3, 3, 3, 2, 2, 3 };
+const unsigned int common_len[] = { 0, 2, 3, 2, 0, 0, 1, 1 };
 
 char usagestr[] = "usage: aucat [-dn] [-b size] "
     "[-c min:max] [-e enc] [-f device] [-g position]\n\t"
     "[-h fmt] [-i file] [-j flag] [-o file] [-p position] [-q port]\n\t"
     "[-r rate] [-v volume]\n";
 
+static void *
+allocbuf(int nfr, int nch)
+{
+	size_t fsize;
+
+	if (nch < 0 || nch > NCHAN_MAX) {
+		log_puts("allocbuf: bogus channel count\n");
+		panic();
+	}
+	fsize = nch * sizeof(adata_t);
+	return xmalloc(nfr * fsize);
+}
+
 static void
 slot_log(struct slot *s)
 {
@@ -278,7 +291,8 @@ slot_init(struct slot *s)
 	}
 #endif
 	s->bpf = s->afile.par.bps * (s->cmax - s->cmin + 1);
-	s->round = (dev_round * s->afile.rate + dev_rate - 1) / dev_rate;
+	s->round = ((long long)dev_round * s->afile.rate +
+	    dev_rate - 1) / dev_rate;
 
 	bufsz = s->round * (dev_bufsz / dev_round);
 	bufsz -= bufsz % s->round;
@@ -314,14 +328,12 @@ slot_init(struct slot *s)
 		if (s->afile.fmt != AFILE_FMT_PCM ||
 		    !aparams_native(&s->afile.par)) {
 			dec_init(&s->conv, &s->afile.par, slot_nch);
-			s->convbuf =
-			    xmalloc(s->round * slot_nch * sizeof(adata_t));
+			s->convbuf = allocbuf(s->round, slot_nch);
 		}
 		if (s->afile.rate != dev_rate) {
 			resamp_init(&s->resamp, s->afile.rate, dev_rate,
 			    slot_nch);
-			s->resampbuf =
-			    xmalloc(dev_round * slot_nch * sizeof(adata_t));
+			s->resampbuf = allocbuf(dev_round, slot_nch);
 		}
 	}
 	if (s->mode & SIO_REC) {
@@ -339,13 +351,11 @@ slot_init(struct slot *s)
 		if (s->afile.rate != dev_rate) {
 			resamp_init(&s->resamp, dev_rate, s->afile.rate,
 			    slot_nch);
-			s->resampbuf =
-			    xmalloc(dev_round * slot_nch * sizeof(adata_t));
+			s->resampbuf = allocbuf(dev_round, slot_nch);
 		}
 		if (!aparams_native(&s->afile.par)) {
 			enc_init(&s->conv, &s->afile.par, slot_nch);
-			s->convbuf =
-			    xmalloc(s->round * slot_nch * sizeof(adata_t));
+			s->convbuf = allocbuf(s->round, slot_nch);
 		}
 
 		/*
@@ -709,11 +719,11 @@ dev_open(char *dev, int mode, int bufsz, char *port)
 	dev_round = par.round;
 	if (mode & SIO_PLAY) {
 		dev_pchan = par.pchan;
-		dev_pbuf = xmalloc(sizeof(adata_t) * dev_pchan * dev_round);
+		dev_pbuf = allocbuf(dev_round, dev_pchan);
 	}
 	if (mode & SIO_REC) {
 		dev_rchan = par.rchan;
-		dev_rbuf = xmalloc(sizeof(adata_t) * dev_rchan * dev_round);
+		dev_rbuf = allocbuf(dev_round, dev_rchan);
 	}
 	dev_pstate = DEV_STOP;
 	if (log_level >= 2) {
@@ -1072,7 +1082,7 @@ offline(void)
 	dev_bufsz = rate;
 	dev_round = rate;
 	dev_pchan = dev_rchan = cmax + 1;
-	dev_pbuf = dev_rbuf = xmalloc(sizeof(adata_t) * dev_pchan * dev_round);
+	dev_pbuf = dev_rbuf = allocbuf(dev_round, dev_pchan);
 	dev_pstate = DEV_STOP;
 	for (s = slot_list; s != NULL; s = s->next)
 		slot_init(s);
@@ -1165,6 +1175,7 @@ playrec(char *dev, int mode, int bufsz, char *port)
 	if (dev_mh)
 		n += mio_nfds(dev_mh);
 	pfds = xmalloc(n * sizeof(struct pollfd));
+
 	for (s = slot_list; s != NULL; s = s->next)
 		slot_init(s);
 	if (dev_mh == NULL)
@@ -1195,7 +1206,7 @@ playrec(char *dev, int mode, int bufsz, char *port)
 			nm = mio_pollfd(dev_mh, pfds + ns, POLLIN);
 		else
 			nm = 0;
-		if (poll(pfds, ns + nm, -1) < 0) {
+		if (poll(pfds, ns + nm, -1) == -1) {
 			if (errno == EINTR)
 				continue;
 			log_puts("poll failed\n");
