@@ -50,13 +50,13 @@ random_bytes(unsigned char *buf, int len)
 	int fd;
 
 	fd = open(DEV_RANDOM, O_RDONLY);
-	if (fd < 0) {
+	if (fd == -1) {
 		DPERROR(DEV_RANDOM);
 		return 0;
 	}
 	while (len > 0) {
 		n = read(fd, buf, len);
-		if (n < 0) {
+		if (n == -1) {
 			if (errno == EINTR)
 				continue;
 			DPERROR(DEV_RANDOM);
@@ -92,7 +92,7 @@ _aucat_rmsg(struct aucat *hdl, int *eof)
 	while (hdl->rtodo > 0) {
 		data = (unsigned char *)&hdl->rmsg;
 		data += sizeof(struct amsg) - hdl->rtodo;
-		while ((n = read(hdl->fd, data, hdl->rtodo)) < 0) {
+		while ((n = read(hdl->fd, data, hdl->rtodo)) == -1) {
 			if (errno == EINTR)
 				continue;
 			if (errno != EAGAIN) {
@@ -138,7 +138,7 @@ _aucat_wmsg(struct aucat *hdl, int *eof)
 	while (hdl->wtodo > 0) {
 		data = (unsigned char *)&hdl->wmsg;
 		data += sizeof(struct amsg) - hdl->wtodo;
-		while ((n = write(hdl->fd, data, hdl->wtodo)) < 0) {
+		while ((n = write(hdl->fd, data, hdl->wtodo)) == -1) {
 			if (errno == EINTR)
 				continue;
 			if (errno != EAGAIN) {
@@ -170,7 +170,7 @@ _aucat_rdata(struct aucat *hdl, void *buf, size_t len, int *eof)
 	}
 	if (len > hdl->rtodo)
 		len = hdl->rtodo;
-	while ((n = read(hdl->fd, buf, len)) < 0) {
+	while ((n = read(hdl->fd, buf, len)) == -1) {
 		if (errno == EINTR)
 			continue;
 		if (errno != EAGAIN) {
@@ -223,7 +223,7 @@ _aucat_wdata(struct aucat *hdl, const void *buf, size_t len,
 		DPRINTF("_aucat_wdata: len == 0\n");
 		abort();
 	}
-	while ((n = write(hdl->fd, buf, len)) < 0) {
+	while ((n = write(hdl->fd, buf, len)) == -1) {
 		if (errno == EINTR)
 			continue;
 		if (errno != EAGAIN) {
@@ -269,12 +269,12 @@ aucat_mkcookie(unsigned char *cookie)
 	memcpy(path + home_len, COOKIE_SUFFIX, sizeof(COOKIE_SUFFIX));
 	path_len = home_len + sizeof(COOKIE_SUFFIX) - 1;
 	fd = open(path, O_RDONLY);
-	if (fd < 0) {
+	if (fd == -1) {
 		if (errno != ENOENT)
 			DPERROR(path);
 		goto bad_gen;
 	}
-	if (fstat(fd, &sb) < 0) {
+	if (fstat(fd, &sb) == -1) {
 		DPERROR(path);
 		goto bad_close;
 	}
@@ -283,7 +283,7 @@ aucat_mkcookie(unsigned char *cookie)
 		goto bad_close;
 	}
 	len = read(fd, cookie, AMSG_COOKIELEN);
-	if (len < 0) {
+	if (len == -1) {
 		DPERROR(path);
 		goto bad_close;
 	}
@@ -322,25 +322,25 @@ bad_gen:
 	/* create ~/.sndio directory */
 	memcpy(tmp, home, home_len);
 	memcpy(tmp + home_len, COOKIE_DIR, sizeof(COOKIE_DIR));
-	if (mkdir(tmp, 0755) < 0 && errno != EEXIST)
+	if (mkdir(tmp, 0755) == -1 && errno != EEXIST)
 		goto done;
 
 	/* create cookie file in it */
 	memcpy(tmp, path, path_len);
 	memcpy(tmp + path_len, TEMPL_SUFFIX, sizeof(TEMPL_SUFFIX));
 	fd = mkstemp(tmp);
-	if (fd < 0) {
+	if (fd == -1) {
 		DPERROR(tmp);
 		goto done;
 	}
-	if (write(fd, cookie, AMSG_COOKIELEN) < 0) {
+	if (write(fd, cookie, AMSG_COOKIELEN) == -1) {
 		DPERROR(tmp);
 		unlink(tmp);
 		close(fd);
 		goto done;
 	}
 	close(fd);
-	if (rename(tmp, path) < 0) {
+	if (rename(tmp, path) == -1) {
 		DPERROR(tmp);
 		unlink(tmp);
 	}
@@ -370,12 +370,12 @@ aucat_connect_tcp(struct aucat *hdl, char *host, unsigned int unit)
 	for (ai = ailist; ai != NULL; ai = ai->ai_next) {
 		s = socket(ai->ai_family, ai->ai_socktype | SOCK_CLOEXEC,
 		    ai->ai_protocol);
-		if (s < 0) {
+		if (s == -1) {
 			DPERROR("socket");
 			continue;
 		}
 #ifndef HAVE_SOCK_CLOEXEC
-		if (fcntl(s, F_SETFL, FD_CLOEXEC) < 0) {
+		if (fcntl(s, F_SETFL, FD_CLOEXEC) == -1) {
 			DPERROR("FD_CLOEXEC");
 			close(s);
 			s = - 1;
@@ -383,7 +383,7 @@ aucat_connect_tcp(struct aucat *hdl, char *host, unsigned int unit)
 		}
 #endif
 	restart:
-		if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
+		if (connect(s, ai->ai_addr, ai->ai_addrlen) == -1) {
 			if (errno == EINTR)
 				goto restart;
 			DPERROR("connect");
@@ -394,10 +394,10 @@ aucat_connect_tcp(struct aucat *hdl, char *host, unsigned int unit)
 		break;
 	}
 	freeaddrinfo(ailist);
-	if (s < 0)
+	if (s == -1)
 		return 0;
 	opt = 1;
-	if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(int)) < 0) {
+	if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(int)) == -1) {
 		DPERROR("setsockopt");
 		close(s);
 		return 0;
@@ -419,23 +419,23 @@ aucat_connect_un(struct aucat *hdl, unsigned int unit)
 	    SOCKPATH_DIR "-%u/" SOCKPATH_FILE "%u", uid, unit);
 	ca.sun_family = AF_UNIX;
 	s = socket(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0);
-	if (s < 0)
+	if (s == -1)
 		return 0;
 #ifndef HAVE_SOCK_CLOEXEC
-	if (fcntl(s, F_SETFL, FD_CLOEXEC) < 0) {
+	if (fcntl(s, F_SETFL, FD_CLOEXEC) == -1) {
 		DPERROR("FD_CLOEXEC");
 		close(s);
 		return 0;
 	}
 #endif
-	while (connect(s, (struct sockaddr *)&ca, len) < 0) {
+	while (connect(s, (struct sockaddr *)&ca, len) == -1) {
 		if (errno == EINTR)
 			continue;
 		DPERROR(ca.sun_path);
 		/* try shared server */
 		snprintf(ca.sun_path, sizeof(ca.sun_path),
 		    SOCKPATH_DIR "/" SOCKPATH_FILE "%u", unit);
-		while (connect(s, (struct sockaddr *)&ca, len) < 0) {
+		while (connect(s, (struct sockaddr *)&ca, len) == -1) {
 			if (errno == EINTR)
 				continue;
 			DPERROR(ca.sun_path);
@@ -449,27 +449,6 @@ aucat_connect_un(struct aucat *hdl, unsigned int unit)
 	return 1;
 }
 
-static const char *
-parsedev(const char *str, unsigned int *rval)
-{
-	const char *p = str;
-	unsigned int val;
-
-	for (val = 0; *p >= '0' && *p <= '9'; p++) {
-		val = 10 * val + (*p - '0');
-		if (val >= 16) {
-			DPRINTF("%s: number too large\n", str);
-			return NULL;
-		}
-	}
-	if (p == str) {
-		DPRINTF("%s: number expected\n", str);
-		return NULL;
-	}
-	*rval = val;
-	return p;
-}
-
 static const char *
 parsestr(const char *str, char *rstr, unsigned int max)
 {
@@ -516,7 +495,7 @@ _aucat_open(struct aucat *hdl, const char *str, unsigned int mode)
 	} else
 		*host = '\0';
 	if (*p == ',') {
-		p = parsedev(++p, &unit);
+		p = _sndio_parsenum(++p, &unit, 15);
 		if (p == NULL)
 			return 0;
 	} else
@@ -525,7 +504,7 @@ _aucat_open(struct aucat *hdl, const char *str, unsigned int mode)
 		DPRINTF("%s: '/' expected\n", str);
 		return 0;
 	}
-	p = parsedev(++p, &devnum);
+	p = _sndio_parsenum(++p, &devnum, 15);
 	if (p == NULL)
 		return 0;
 	if (*p == '.') {
@@ -569,6 +548,7 @@ _aucat_open(struct aucat *hdl, const char *str, unsigned int mode)
 	hdl->wmsg.u.hello.version = AMSG_VERSION;
 	hdl->wmsg.u.hello.mode = htons(mode);
 	hdl->wmsg.u.hello.devnum = devnum;
+	hdl->wmsg.u.hello.id = htonl(getpid());
 	strlcpy(hdl->wmsg.u.hello.who, __progname,
 	    sizeof(hdl->wmsg.u.hello.who));
 	strlcpy(hdl->wmsg.u.hello.opt, opt,
@@ -587,7 +567,7 @@ _aucat_open(struct aucat *hdl, const char *str, unsigned int mode)
 	}
 	return 1;
  bad_connect:
-	while (close(hdl->fd) < 0 && errno == EINTR)
+	while (close(hdl->fd) == -1 && errno == EINTR)
 		; /* retry */
 	return 0;
 }
@@ -595,7 +575,8 @@ _aucat_open(struct aucat *hdl, const char *str, unsigned int mode)
 void
 _aucat_close(struct aucat *hdl, int eof)
 {
-	char dummy[1];
+	char dummy[sizeof(struct amsg)];
+	ssize_t n;
 
 	if (!eof) {
 		AMSG_INIT(&hdl->wmsg);
@@ -603,18 +584,30 @@ _aucat_close(struct aucat *hdl, int eof)
 		hdl->wtodo = sizeof(struct amsg);
 		if (!_aucat_wmsg(hdl, &eof))
 			goto bad_close;
-		while (read(hdl->fd, dummy, 1) < 0 && errno == EINTR)
-			; /* nothing */
+
+		/*
+		 * block until the peer disconnects
+		 */
+		while (1) {
+			n = read(hdl->fd, dummy, sizeof(dummy));
+			if (n == -1) {
+				if (errno == EINTR)
+					continue;
+				break;
+			}
+			if (n == 0)
+				break;
+		}
 	}
  bad_close:
-	while (close(hdl->fd) < 0 && errno == EINTR)
+	while (close(hdl->fd) == -1 && errno == EINTR)
 		; /* nothing */
 }
 
 int
 _aucat_setfl(struct aucat *hdl, int nbio, int *eof)
 {
-	if (fcntl(hdl->fd, F_SETFL, nbio ? O_NONBLOCK : 0) < 0) {
+	if (fcntl(hdl->fd, F_SETFL, nbio ? O_NONBLOCK : 0) == -1) {
 		DPERROR("_aucat_setfl: fcntl");
 		*eof = 1;
 		return 0;
