@@ -16,7 +16,6 @@
  */
 
 #include <sys/types.h>
-#include <sys/stat.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -54,7 +53,12 @@ sio_open(const char *str, unsigned int mode, int nbio)
 	if (str == NULL) /* backward compat */
 		str = devany;
 	if (strcmp(str, devany) == 0 && !issetugid()) {
-		str = getenv("AUDIODEVICE");
+		if ((mode & SIO_PLAY) == 0)
+			str = getenv("AUDIORECDEVICE");
+		if ((mode & SIO_REC) == 0)
+			str = getenv("AUDIOPLAYDEVICE");
+		if (mode == (SIO_PLAY | SIO_REC) || str == NULL)
+			str = getenv("AUDIODEVICE");
 		if (str == NULL)
 			str = devany;
 	}
@@ -63,7 +67,7 @@ sio_open(const char *str, unsigned int mode, int nbio)
 		if (hdl != NULL)
 			return hdl;
 #if defined(USE_SUN)
-		return _sio_sun_open("rsnd/default", mode, nbio);
+		return _sio_sun_open("rsnd/0", mode, nbio);
 #elif defined(USE_OSS)
 		return _sio_oss_open("rsnd/default", mode, nbio);
 #elif defined(USE_ALSA)
@@ -238,7 +242,7 @@ sio_psleep(struct sio_hdl *hdl, int event)
 	}
 	for (;;) {
 		nfds = sio_pollfd(hdl, pfd, event);
-		while (poll(pfd, nfds, -1) < 0) {
+		while (poll(pfd, nfds, -1) == -1) {
 			if (errno == EINTR)
 				continue;
 			DPERROR("sio_psleep: poll");
@@ -312,10 +316,6 @@ sio_read(struct sio_hdl *hdl, void *buf, size_t len)
 		hdl->eof = 1;
 		return 0;
 	}
-	if (todo == 0) {
-		DPRINTF("sio_read: zero length read ignored\n");
-		return 0;
-	}
 	while (todo > 0) {
 		if (!sio_rdrop(hdl))
 			return 0;
@@ -353,10 +353,6 @@ sio_write(struct sio_hdl *hdl, const void *buf, size_t len)
 		hdl->eof = 1;
 		return 0;
 	}
-	if (todo == 0) {
-		DPRINTF("sio_write: zero length write ignored\n");
-		return 0;
-	}
 	while (todo > 0) {
 		if (!sio_wsil(hdl))
 			return 0;
