@@ -489,8 +489,10 @@ sio_alsa_xrun(struct sio_alsa_hdl *hdl)
 	int wbpf;
 
 	DPRINTFN(2, "sio_alsa_xrun:\n");
+#ifdef DEBUG
 	if (_sndio_debug >= 2)
 		_sio_printpos(&hdl->sio);
+#endif
 
 	/*
 	 * we assume rused/wused are zero if rec/play modes are not
@@ -552,6 +554,11 @@ sio_alsa_setpar_hw(snd_pcm_t *pcm, snd_pcm_hw_params_t *hwp,
 
 	req_rate = *rate;
 
+	err = snd_pcm_hw_free(pcm);
+	if (err < 0) {
+		DALSA("couldn't reset hw configuration", err);
+		return 0;
+	}
 	err = snd_pcm_hw_params_any(pcm, hwp);
 	if (err < 0) {
 		DALSA("couldn't init pars", err);
@@ -819,9 +826,9 @@ sio_alsa_setpar(struct sio_hdl *sh, struct sio_par *par)
 		return 0;
 	hdl->par.msb = 1;
 	hdl->par.bps = SIO_BPS(hdl->par.bits);
-	hdl->par.rate = irate;
-	hdl->par.round = iround;
-	hdl->par.bufsz = iround * iperiods;
+	hdl->par.rate = orate;
+	hdl->par.round = oround;
+	hdl->par.bufsz = oround * operiods;
 	hdl->par.appbufsz = hdl->par.bufsz;
 
 	/* software params */
@@ -875,7 +882,7 @@ sio_alsa_setpar(struct sio_hdl *sh, struct sio_par *par)
 			return 0;
 		}
 		err = snd_pcm_sw_params_set_start_threshold(hdl->opcm,
-		    oswp, hdl->par.bufsz);
+		    oswp, hdl->par.bufsz - hdl->par.round);
 		if (err < 0) {
 			DALSA("couldn't set play start threshold", err);
 			hdl->sio.eof = 1;
