@@ -19,7 +19,6 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/audioio.h>
-#include <sys/stat.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -82,12 +81,12 @@ sio_sun_adjpar(struct sio_sun_hdl *hdl, struct audio_swpar *ap)
 {
 	if (hdl->sio.eof)
 		return 0;
-	if (ioctl(hdl->fd, AUDIO_SETPAR, ap)) {
+	if (ioctl(hdl->fd, AUDIO_SETPAR, ap) == -1) {
 		DPERROR("AUDIO_SETPAR");
 		hdl->sio.eof = 1;
 		return 0;
 	}
-	if (ioctl(hdl->fd, AUDIO_GETPAR, ap)) {
+	if (ioctl(hdl->fd, AUDIO_GETPAR, ap) == -1) {
 		DPERROR("AUDIO_GETPAR");
 		hdl->sio.eof = 1;
 		return 0;
@@ -166,7 +165,7 @@ sio_sun_getcap(struct sio_hdl *sh, struct sio_cap *cap)
 	unsigned int enc_map = 0, rchan_map = 0, pchan_map = 0, rate_map;
 	unsigned int i, j, conf;
 
-	if (ioctl(hdl->fd, AUDIO_GETPAR, &savepar)) {
+	if (ioctl(hdl->fd, AUDIO_GETPAR, &savepar) == -1) {
 		DPERROR("AUDIO_GETPAR");
 		hdl->sio.eof = 1;
 		return 0;
@@ -259,7 +258,7 @@ sio_sun_getcap(struct sio_hdl *sh, struct sio_cap *cap)
 	}
 	cap->nconf = nconf;
 
-	if (ioctl(hdl->fd, AUDIO_SETPAR, &savepar)) {
+	if (ioctl(hdl->fd, AUDIO_SETPAR, &savepar) == -1) {
 		DPERROR("AUDIO_SETPAR");
 		hdl->sio.eof = 1;
 		return 0;
@@ -291,21 +290,17 @@ sio_sun_getfd(const char *str, unsigned int mode, int nbio)
 		DPRINTF("sio_sun_getfd: %s: '/' expected\n", str);
 		return -1;
 	}
-	if (strcmp(p, "default") == 0) {
-		devnum = 0;
-	} else {
-		p = _sndio_parsenum(p, &devnum, 255);
-		if (p == NULL || *p != '\0') {
-			DPRINTF("sio_sun_getfd: %s: number expected after '/'\n", str);
-			return -1;
-		}
+	p = _sndio_parsenum(p, &devnum, 255);
+	if (p == NULL || *p != '\0') {
+		DPRINTF("sio_sun_getfd: %s: number expected after '/'\n", str);
+		return -1;
 	}
 	snprintf(path, sizeof(path), DEVPATH_PREFIX "%u", devnum);
 	if (mode == (SIO_PLAY | SIO_REC))
 		flags = O_RDWR;
 	else
 		flags = (mode & SIO_PLAY) ? O_WRONLY : O_RDONLY;
-	while ((fd = open(path, flags | O_NONBLOCK | O_CLOEXEC)) < 0) {
+	while ((fd = open(path, flags | O_NONBLOCK | O_CLOEXEC)) == -1) {
 		if (errno == EINTR)
 			continue;
 		DPERROR(path);
@@ -330,7 +325,7 @@ sio_sun_fdopen(int fd, unsigned int mode, int nbio)
 	/*
 	 * pause the device
 	 */
-	if (ioctl(fd, AUDIO_STOP) < 0) {
+	if (ioctl(fd, AUDIO_STOP) == -1) {
 		DPERROR("AUDIO_STOP");
 		free(hdl);
 		return NULL;
@@ -347,12 +342,12 @@ _sio_sun_open(const char *str, unsigned int mode, int nbio)
 	int fd;
 
 	fd = sio_sun_getfd(str, mode, nbio);
-	if (fd < 0)
+	if (fd == -1)
 		return NULL;
 	hdl = sio_sun_fdopen(fd, mode, nbio);
 	if (hdl != NULL)
 		return hdl;
-	while (close(fd) < 0 && errno == EINTR)
+	while (close(fd) == -1 && errno == EINTR)
 		; /* retry */
 	return NULL;
 }
@@ -362,7 +357,7 @@ sio_sun_close(struct sio_hdl *sh)
 {
 	struct sio_sun_hdl *hdl = (struct sio_sun_hdl *)sh;
 
-	while (close(hdl->fd) < 0 && errno == EINTR)
+	while (close(hdl->fd) == -1 && errno == EINTR)
 		; /* retry */
 	free(hdl);
 }
@@ -391,7 +386,7 @@ sio_sun_start(struct sio_hdl *sh)
 		/*
 		 * no play buffers to fill, start now!
 		 */
-		if (ioctl(hdl->fd, AUDIO_START) < 0) {
+		if (ioctl(hdl->fd, AUDIO_START) == -1) {
 			DPERROR("AUDIO_START");
 			hdl->sio.eof = 1;
 			return 0;
@@ -410,7 +405,7 @@ sio_sun_stop(struct sio_hdl *sh)
 		hdl->filling = 0;
 		return 1;
 	}
-	if (ioctl(hdl->fd, AUDIO_STOP) < 0) {
+	if (ioctl(hdl->fd, AUDIO_STOP) == -1) {
 		DPERROR("AUDIO_STOP");
 		hdl->sio.eof = 1;
 		return 0;
@@ -445,7 +440,7 @@ sio_sun_setpar(struct sio_hdl *sh, struct sio_par *par)
 		ap.round = par->appbufsz / 2;
 		ap.nblks = 2;
 	}
-	if (ioctl(hdl->fd, AUDIO_SETPAR, &ap) < 0) {
+	if (ioctl(hdl->fd, AUDIO_SETPAR, &ap) == -1) {
 		DPERROR("AUDIO_SETPAR");
 		hdl->sio.eof = 1;
 		return 0;
@@ -459,7 +454,7 @@ sio_sun_getpar(struct sio_hdl *sh, struct sio_par *par)
 	struct sio_sun_hdl *hdl = (struct sio_sun_hdl *)sh;
 	struct audio_swpar ap;
 
-	if (ioctl(hdl->fd, AUDIO_GETPAR, &ap) < 0) {
+	if (ioctl(hdl->fd, AUDIO_GETPAR, &ap) == -1) {
 		DPERROR("AUDIO_GETPAR");
 		hdl->sio.eof = 1;
 		return 0;
@@ -484,7 +479,7 @@ sio_sun_read(struct sio_hdl *sh, void *buf, size_t len)
 	struct sio_sun_hdl *hdl = (struct sio_sun_hdl *)sh;
 	ssize_t n;
 
-	while ((n = read(hdl->fd, buf, len)) < 0) {
+	while ((n = read(hdl->fd, buf, len)) == -1) {
 		if (errno == EINTR)
 			continue;
 		if (errno != EAGAIN) {
@@ -509,7 +504,7 @@ sio_sun_write(struct sio_hdl *sh, const void *buf, size_t len)
 	ssize_t n, todo;
 
 	todo = len;
-	while ((n = write(hdl->fd, data, todo)) < 0) {
+	while ((n = write(hdl->fd, data, todo)) == -1) {
 		if (errno == EINTR)
 			continue;
 		if (errno != EAGAIN) {
@@ -537,7 +532,7 @@ sio_sun_pollfd(struct sio_hdl *sh, struct pollfd *pfd, int events)
 	if (hdl->filling && hdl->sio.wused == hdl->sio.par.bufsz *
 		hdl->sio.par.pchan * hdl->sio.par.bps) {
 		hdl->filling = 0;
-		if (ioctl(hdl->fd, AUDIO_START) < 0) {
+		if (ioctl(hdl->fd, AUDIO_START) == -1) {
 			DPERROR("AUDIO_START");
 			hdl->sio.eof = 1;
 			return 0;
@@ -558,7 +553,7 @@ sio_sun_revents(struct sio_hdl *sh, struct pollfd *pfd)
 	if ((pfd->revents & POLLHUP) ||
 	    (pfd->revents & (POLLIN | POLLOUT)) == 0)
 		return pfd->revents;
-	if (ioctl(hdl->fd, AUDIO_GETPOS, &ap) < 0) {
+	if (ioctl(hdl->fd, AUDIO_GETPOS, &ap) == -1) {
 		DPERROR("sio_sun_revents: GETPOS");
 		hdl->sio.eof = 1;
 		return POLLHUP;
