@@ -20,6 +20,18 @@
 #include "abuf.h"
 #include "dsp.h"
 #include "siofile.h"
+#include "dev_sioctl.h"
+#include "opt.h"
+
+/*
+ * preallocated audio clients
+ */
+#define DEV_NSLOT	8
+
+/*
+ * preallocated control clients
+ */
+#define DEV_NCTLSLOT 8
 
 /*
  * audio stream state structure
@@ -28,17 +40,22 @@
 struct slotops
 {
 	void (*onmove)(void *);			/* clock tick */
-	void (*onvol)(void *);	        /* tell client vol changed */
+	void (*onvol)(void *);			/* tell client vol changed */
 	void (*fill)(void *);			/* request to fill a play block */
 	void (*flush)(void *);			/* request to flush a rec block */
 	void (*eof)(void *);			/* notify that play drained */
 	void (*exit)(void *);			/* delete client */
 };
 
+struct ctlops
+{
+	void (*exit)(void *);			/* delete client */
+	void (*sync)(void *);			/* description ready */
+};
+
 struct slot {
 	struct slotops *ops;			/* client callbacks */
 	struct slot *next;			/* next on the play list */
-	struct dev *dev;			/* device this belongs to */
 	struct opt *opt;			/* config used */
 	void *arg;				/* user data for callbacks */
 	struct aparams par;			/* socket side params */
@@ -70,7 +87,7 @@ struct slot {
 	int xrun;				/* underrun policy */
 	int skip;				/* cycles to skip (for xrun) */
 #define SLOT_BUFSZ(s) \
-	((s)->appbufsz + (s)->dev->bufsz / (s)->dev->round * (s)->round)
+	((s)->appbufsz + (s)->opt->dev->bufsz / (s)->opt->dev->round * (s)->round)
 	int appbufsz;				/* slot-side buffer size */
 	int round;				/* slot-side block size */
 	int rate;				/* slot-side sample rate */
@@ -89,18 +106,102 @@ struct slot {
 	unsigned int unit;			/* instance of name */
 	unsigned int serial;			/* global unique number */
 	unsigned int vol;			/* current (midi) volume */
+	unsigned int id;			/* process id */
 };
 
-struct opt {
-	struct opt *next;
-#define OPT_NAMEMAX 11
-	char name[OPT_NAMEMAX + 1];
-	int maxweight;		/* max dynamic range for clients */
-	int pmin, pmax;		/* play channels */
-	int rmin, rmax;		/* recording channels */
-	int mmc;		/* true if MMC control enabled */
-	int dup;		/* true if join/expand enabled */
-	int mode;		/* bitmap of MODE_XXX */
+/*
+ * subset of channels of a stream
+ */
+
+struct ctl {
+	struct ctl *next;
+
+#define CTL_NONE	0		/* deleted */
+#define CTL_NUM		2		/* number (aka integer value) */
+#define CTL_SW		3		/* on/off switch, only bit 7 counts */
+#define CTL_VEC		4		/* number, element of vector */
+#define CTL_LIST	5		/* switch, element of a list */
+#define CTL_SEL		6		/* element of a selector */
+	unsigned int type;		/* one of above */
+
+#define CTL_HW		0
+#define CTL_DEV_MASTER	1
+#define CTL_DEV_ALT	2
+#define CTL_SLOT_LEVEL	3
+	unsigned int scope;
+	union {
+		struct {
+			void *arg0;
+			void *arg1;
+		} any;
+		struct {
+			struct dev *dev;
+			unsigned int addr;
+		} hw;
+		struct {
+			struct dev *dev;
+		} dev_master;
+		struct {
+			struct dev *dev;
+			unsigned int idx;
+		} dev_alt;
+		struct {
+			struct slot *slot;
+		} slot_level;
+	} u;
+
+	unsigned int addr;		/* slot side control address */
+#define CTL_NAMEMAX	16		/* max name lenght */
+	char func[CTL_NAMEMAX];		/* parameter function name */
+	char group[CTL_NAMEMAX];	/* group aka namespace */
+	struct ctl_node {
+		char name[CTL_NAMEMAX];	/* stream name */
+		int unit;
+	} node0, node1;			/* affected channels */
+#define CTL_DEVMASK		(1 << 31)
+#define CTL_SLOTMASK(i)		(1 << (i))
+	unsigned int val_mask;
+	unsigned int desc_mask;
+	unsigned int refs_mask;
+	unsigned int maxval;
+	unsigned int curval;
+	int dirty;
+};
+
+struct ctlslot {
+	struct ctlops *ops;
+	void *arg;
+	struct opt *opt;
+	unsigned int self;		/* equal to (1 << index) */
+	unsigned int mode;
+};
+
+/*
+ * MIDI time code (MTC)
+ */
+struct mtc {
+	/*
+	 * MIDI time code (MTC) states
+	 */
+#define MTC_STOP	1		/* stopped, can't start */
+#define MTC_START	2		/* attempting to start */
+#define MTC_RUN		3		/* started */
+	unsigned int tstate;		/* one of MTC_* constants */
+	struct dev *dev;
+
+	unsigned int origin;		/* MTC start time */
+	unsigned int fps;		/* MTC frames per second */
+#define MTC_FPS_24	0
+#define MTC_FPS_25	1
+#define MTC_FPS_30	3
+	unsigned int fps_id;		/* one of above */
+	unsigned int hr;		/* MTC hours */
+	unsigned int min;		/* MTC minutes */
+	unsigned int sec;		/* MTC seconds */
+	unsigned int fr;		/* MTC frames */
+	unsigned int qfr;		/* MTC quarter frames */
+	int delta;			/* rel. to the last MTC tick */
+	int refs;
 };
 
 /*
@@ -109,13 +210,17 @@ struct opt {
 struct dev {
 	struct dev *next;
 	struct slot *slot_list;			/* audio streams attached */
-	struct opt *opt_list;
-	struct midi *midi;
+
+	/*
+	 * name used for various controls
+	 */
+	char name[CTL_NAMEMAX];
 
 	/*
 	 * audio device (while opened)
 	 */
 	struct dev_sio sio;
+	struct dev_sioctl sioctl;
 	struct aparams par;			/* encoding */
 	int pchan, rchan;			/* play & rec channels */
 	adata_t *rbuf;				/* rec buffer */
@@ -128,13 +233,6 @@ struct dev {
 	unsigned char *encbuf;			/* buffer for encoding */
 	unsigned char *decbuf;			/* buffer for decoding */
 
-	/*
-	 * preallocated audio sub-devices
-	 */
-#define DEV_NSLOT	8
-	struct slot slot[DEV_NSLOT];
-	unsigned int serial;			/* for slot allocation */
-
 	/*
 	 * current position, relative to the current cycle
 	 */
@@ -158,7 +256,12 @@ struct dev {
 #define DEV_INIT	1			/* stopped */
 #define DEV_RUN		2			/* playin & recording */
 	unsigned int pstate;			/* one of above */
-	char *path;				/* sio path */
+	struct dev_alt {
+		struct dev_alt *next;
+		char *name;
+		unsigned int idx;
+	} *alt_list;
+	int alt_num;
 
 	/*
 	 * actual parameters and runtime state (i.e. once opened)
@@ -167,42 +270,25 @@ struct dev {
 	unsigned int bufsz, round, rate;
 	unsigned int prime;
 
-	/*
-	 * MIDI time code (MTC)
-	 */
-	struct {
-		unsigned int origin;		/* MTC start time */
-		unsigned int fps;		/* MTC frames per second */
-#define MTC_FPS_24	0
-#define MTC_FPS_25	1
-#define MTC_FPS_30	3
-		unsigned int fps_id;		/* one of above */
-		unsigned int hr;		/* MTC hours */
-		unsigned int min;		/* MTC minutes */
-		unsigned int sec;		/* MTC seconds */
-		unsigned int fr;		/* MTC frames */
-		unsigned int qfr;		/* MTC quarter frames */
-		int delta;			/* rel. to the last MTC tick */
-		int refs;
-	} mtc;
-
-	/*
-	 * MIDI machine control (MMC)
-	 */
-#define MMC_STOP	1			/* stopped, can't start */
-#define MMC_START	2			/* attempting to start */
-#define MMC_RUN		3			/* started */
-	unsigned int tstate;			/* one of above */
-	unsigned int master;			/* master volume controller */
+	unsigned int master;			/* software vol. knob */
+	unsigned int master_enabled;		/* 1 if h/w has no vo. knob */
 };
 
 extern struct dev *dev_list;
+extern struct ctl *ctl_list;
+extern struct slot slot_array[DEV_NSLOT];
+extern struct ctlslot ctlslot_array[DEV_NCTLSLOT];
+extern struct mtc mtc_array[1];
+
+void slot_array_init(void);
 
 void dev_log(struct dev *);
-void dev_close(struct dev *);
+void dev_abort(struct dev *);
+int dev_reopen(struct dev *);
 struct dev *dev_new(char *, struct aparams *, unsigned int, unsigned int,
     unsigned int, unsigned int, unsigned int, unsigned int);
 struct dev *dev_bynum(int);
+int dev_addname(struct dev *, char *);
 void dev_del(struct dev *);
 void dev_adjpar(struct dev *, int, int, int);
 int  dev_init(struct dev *);
@@ -221,23 +307,56 @@ void dev_cycle(struct dev *);
 /*
  * midi & midi call-backs
  */
-void dev_mmcstart(struct dev *);
-void dev_mmcstop(struct dev *);
-void dev_mmcloc(struct dev *, unsigned int);
 void dev_master(struct dev *, unsigned int);
+void dev_midi_send(struct dev *, void *, int);
 void dev_midi_vol(struct dev *, struct slot *);
+void dev_midi_master(struct dev *);
+void dev_midi_slotdesc(struct dev *, struct slot *);
+void dev_midi_dump(struct dev *);
+
+void mtc_midi_qfr(struct mtc *, int);
+void mtc_midi_full(struct mtc *);
+void mtc_trigger(struct mtc *);
+void mtc_start(struct mtc *);
+void mtc_stop(struct mtc *);
+void mtc_loc(struct mtc *, unsigned int);
+void mtc_setdev(struct mtc *, struct dev *);
 
 /*
  * sio_open(3) like interface for clients
  */
 void slot_log(struct slot *);
-struct slot *slot_new(struct dev *, struct opt *, char *,
+struct slot *slot_new(struct opt *, unsigned int, char *,
     struct slotops *, void *, int);
 void slot_del(struct slot *);
 void slot_setvol(struct slot *, unsigned int);
 void slot_start(struct slot *);
-void slot_stop(struct slot *);
+void slot_stop(struct slot *, int);
 void slot_read(struct slot *);
 void slot_write(struct slot *);
+void slot_initconv(struct slot *);
+void slot_attach(struct slot *);
+void slot_detach(struct slot *);
+
+/*
+ * control related functions
+ */
+
+struct ctl *ctl_new(int, void *, void *,
+    int, char *, char *, int, char *, char *, int, int, int);
+void ctl_del(int, void *, void *);
+void ctl_log(struct ctl *);
+int ctl_setval(struct ctl *c, int val);
+int ctl_match(struct ctl *, int, void *, void *);
+struct ctl *ctl_find(int, void *, void *);
+void ctl_update(struct ctl *);
+int ctl_onval(int, void *, void *, int);
+
+struct ctlslot *ctlslot_new(struct opt *, struct ctlops *, void *);
+void ctlslot_del(struct ctlslot *);
+int ctlslot_visible(struct ctlslot *, struct ctl *);
+struct ctl *ctlslot_lookup(struct ctlslot *, int);
+void dev_label(struct dev *, int);
+void dev_ctlsync(struct dev *);
 
 #endif /* !defined(DEV_H) */
