@@ -40,57 +40,43 @@ typedef short adata_t;
 
 #elif ADATA_BITS == 24
 
-#if defined(__i386__) && defined(__GNUC__)
-
-static inline int
-fp24_mul(int x, int a)
-{
-	int res;
-
-	asm volatile (
-		"imull	%2\n\t"
-		"shrdl $23, %%edx, %%eax\n\t"
-		: "=a" (res)
-		: "a" (x), "r" (a)
-		: "%edx"
-		);
-	return res;
-}
-
-static inline int
-fp24_muldiv(int x, int a, int b)
-{
-	int res;
-
-	asm volatile (
-		"imull %2\n\t"
-		"idivl %3\n\t"
-		: "=a" (res)
-		: "a" (x), "d" (a), "r" (b)
-		);
-	return res;
-}
-
-#define ADATA_MUL(x,y)		fp24_mul(x, y)
-#define ADATA_MULDIV(x,y,z)	fp24_muldiv(x, y, z);
-
-#elif defined(__amd64__) || defined(__sparc64__)
-
 #define ADATA_MUL(x,y)		\
 	((int)(((long long)(x) * (long long)(y)) >> (ADATA_BITS - 1)))
 #define ADATA_MULDIV(x,y,z)	\
 	((int)((long long)(x) * (long long)(y) / (long long)(z)))
 
-#else
-#error "no 24-bit code for this architecture"
-#endif
-
 typedef int adata_t;
 
 #else
 #error "only 16-bit and 24-bit precisions are supported"
 #endif
 
+/*
+ * The FIR is sampled and stored in a table of fixed-point numbers
+ * with 23 fractional bits. For convenience, we use the same fixed-point
+ * numbers to represent time and to walk through the table.
+ */
+#define RESAMP_BITS		23
+#define RESAMP_UNIT		(1 << RESAMP_BITS)
+
+/*
+ * Filter window length (the time unit is RESAMP_UNIT)
+ */
+#define RESAMP_LENGTH		(8 * RESAMP_UNIT)
+
+/*
+ * Time between samples of the FIR (the time unit is RESAMP_UNIT)
+ */
+#define RESAMP_STEP_BITS	(RESAMP_BITS - 6)
+#define RESAMP_STEP		(1 << RESAMP_STEP_BITS)
+
+/*
+ * Maximum downsample/upsample ratio we support, must be a power of two.
+ * The ratio between the max and the min sample rates is 192kHz / 4kHz = 48,
+ * so we can use 64
+ */
+#define RESAMP_RATIO		64
+
 /*
  * Maximum size of the encording string (the longest possible
  * encoding is ``s24le3msb'').
@@ -111,9 +97,10 @@ struct aparams {
 };
 
 struct resamp {
-#define RESAMP_NCTX	2
+#define RESAMP_NCTX	(RESAMP_LENGTH / RESAMP_UNIT * RESAMP_RATIO)
 	unsigned int ctx_start;
 	adata_t ctx[NCHAN_MAX * RESAMP_NCTX];
+	int filt_cutoff, filt_step;
 	unsigned int iblksz, oblksz;
 	int nch;
 };
@@ -137,7 +124,7 @@ struct cmap {
 };
 
 #define MIDI_TO_ADATA(m)	(aparams_ctltovol[m] << (ADATA_BITS - 16))
-extern int aparams_ctltovol[128];
+extern const int aparams_ctltovol[128];
 
 void aparams_init(struct aparams *);
 void aparams_log(struct aparams *);
