@@ -33,6 +33,7 @@ void port_imsg(void *, unsigned char *, int);
 void port_omsg(void *, unsigned char *, int);
 void port_fill(void *, int);
 void port_exit(void *);
+void port_exitall(struct port *);
 
 struct midiops port_midiops = {
 	port_imsg,
@@ -54,8 +55,8 @@ struct midithru {
 /*
  * length of voice and common messages (status byte included)
  */
-unsigned int voice_len[] = { 3, 3, 3, 3, 2, 2, 3 };
-unsigned int common_len[] = { 0, 2, 3, 2, 0, 0, 1, 1 };
+const unsigned int voice_len[] = { 3, 3, 3, 3, 2, 2, 3 };
+const unsigned int common_len[] = { 0, 2, 3, 2, 0, 0, 1, 1 };
 
 void
 midi_log(struct midi *ep)
@@ -92,6 +93,7 @@ midi_new(struct midiops *ops, void *arg, int mode)
 	ep->len = 0;
 	ep->idx = 0;
 	ep->st = 0;
+	ep->last_st = 0;
 	ep->txmask = 0;
 	ep->self = 1 << i;
 	ep->tickets = 0;
@@ -188,6 +190,25 @@ midi_tag(struct midi *ep, unsigned int tag)
 		t->txmask |= ep->self;
 }
 
+/*
+ * return the list of tags
+ */
+unsigned int
+midi_tags(struct midi *ep)
+{
+	int i;
+	struct midithru *t;
+	unsigned int tags;
+
+	tags = 0;
+	for (i = 0; i < MIDITHRU_NMAX; i++) {
+		t = midithru + i;
+		if ((t->txmask | t->rxmask) & ep->self)
+			tags |= 1 << i;
+	}
+	return tags;
+}
+
 /*
  * broadcast the given message to other endpoints
  */
@@ -239,6 +260,16 @@ midi_tickets(struct midi *iep)
 	int i, tickets, avail, maxavail;
 	struct midi *oep;
 
+	/*
+	 * don't request iep->ops->fill() too often as it generates
+	 * useless network traffic: wait until we reach half of the
+	 * max tickets count. As in the worst case (see comment below)
+	 * one ticket may consume two bytes, the max ticket count is
+	 * BUFSZ / 2 and halt of it is simply BUFSZ / 4.
+	 */
+	if (iep->tickets >= MIDI_BUFSZ / 4)
+		return;
+
 	maxavail = MIDI_BUFSZ;
 	for (i = 0; i < MIDI_NEP ; i++) {
 		if ((iep->txmask & (1 << i)) == 0)
@@ -295,7 +326,18 @@ midi_in(struct midi *iep, unsigned char *idata, int icount)
 				iep->msg[iep->idx++] = c;
 				iep->ops->imsg(iep->arg, iep->msg, iep->idx);
 			}
-			iep->st = 0;
+
+			/*
+			 * There are bogus MIDI sources that keep
+			 * state across sysex; Linux virmidi ports fed
+			 * by the sequencer is an example. We
+			 * workaround this by saving the current
+			 * status and restoring it at the end of the
+			 * sysex.
+			 */
+			iep->st = iep->last_st;
+			if (iep->st)
+				iep->len = voice_len[(iep->st >> 4) & 7];
 			iep->idx = 0;
 		} else if (c >= 0xf0) {
 			iep->msg[0] = c;
@@ -305,7 +347,7 @@ midi_in(struct midi *iep, unsigned char *idata, int icount)
 		} else if (c >= 0x80) {
 			iep->msg[0] = c;
 			iep->len = voice_len[(c >> 4) & 7];
-			iep->st = c;
+			iep->last_st = iep->st = c;
 			iep->idx = 1;
 		} else if (iep->st) {
 			if (iep->idx == 0 && iep->st != SYSEX_START)
@@ -376,6 +418,22 @@ midi_out(struct midi *oep, unsigned char *idata, int icount)
 	}
 }
 
+/*
+ * disconnect clients attached to this end-point
+ */
+void
+midi_abort(struct midi *p)
+{
+	int i;
+	struct midi *ep;
+
+	for (i = 0; i < MIDI_NEP; i++) {
+		ep = midi_ep + i;
+		if ((ep->txmask & p->self) || (p->txmask & ep->self))
+			ep->ops->exit(ep->arg);
+	}
+}
+
 void
 port_log(struct port *p)
 {
@@ -425,16 +483,19 @@ port_exit(void *arg)
 struct port *
 port_new(char *path, unsigned int mode, int hold)
 {
-	struct port *c;
+	struct port *c, **pc;
 
 	c = xmalloc(sizeof(struct port));
-	c->path = xstrdup(path);
+	c->path_list = NULL;
+	namelist_add(&c->path_list, path);
 	c->state = PORT_CFG;
 	c->hold = hold;
 	c->midi = midi_new(&port_midiops, c, mode);
 	c->num = midi_portnum++;
-	c->next = port_list;
-	port_list = c;
+	for (pc = &port_list; *pc != NULL; pc = &(*pc)->next)
+		;
+	c->next = *pc;
+	*pc = c;
 	return c;
 }
 
@@ -458,7 +519,7 @@ port_del(struct port *c)
 #endif
 	}
 	*p = c->next;
-	xfree(c->path);
+	namelist_clear(&c->path_list);
 	xfree(c);
 }
 
@@ -511,7 +572,7 @@ port_open(struct port *c)
 {
 	if (!port_mio_open(c)) {
 		if (log_level >= 1) {
-			log_puts(c->path);
+			port_log(c);
 			log_puts(": failed to open midi port\n");
 		}
 		return 0;
@@ -523,8 +584,6 @@ port_open(struct port *c)
 int
 port_close(struct port *c)
 {
-	int i;
-	struct midi *ep;
 #ifdef DEBUG
 	if (c->state == PORT_CFG) {
 		port_log(c);
@@ -534,13 +593,6 @@ port_close(struct port *c)
 #endif
 	c->state = PORT_CFG;
 	port_mio_close(c);
-
-	for (i = 0; i < MIDI_NEP; i++) {
-		ep = midi_ep + i;
-		if ((ep->txmask & c->midi->self) ||
-		    (c->midi->txmask & ep->self))
-			ep->ops->exit(ep->arg);
-	}
 	return 1;
 }
 
@@ -576,3 +628,15 @@ port_done(struct port *c)
 	if (c->state == PORT_INIT)
 		port_drain(c);
 }
+
+int
+port_reopen(struct port *p)
+{
+	if (p->state == PORT_CFG)
+		return 1;
+
+	if (!port_mio_reopen(p))
+		return 0;
+
+	return 1;
+}
