@@ -68,7 +68,7 @@
  * block size if neither ``-z'' nor ``-b'' is used
  */
 #ifndef DEFAULT_ROUND
-#define DEFAULT_ROUND	960
+#define DEFAULT_ROUND	480
 #endif
 
 /*
@@ -78,14 +78,8 @@
 #define DEFAULT_BUFSZ	7680
 #endif
 
-/*
- * default device in server mode
- */
-#ifndef DEFAULT_DEV
-#define DEFAULT_DEV "rsnd/default"
-#endif
-
 void sigint(int);
+void sighup(int);
 void opt_ch(int *, int *);
 void opt_enc(struct aparams *);
 int opt_mmc(void);
@@ -102,12 +96,34 @@ struct opt *mkopt(char *, struct dev *,
     int, int, int, int, int, int, int, int);
 
 unsigned int log_level = 0;
-volatile sig_atomic_t quit_flag = 0;
+volatile sig_atomic_t quit_flag = 0, reopen_flag = 0;
 
 char usagestr[] = "usage: sndiod [-d] [-a flag] [-b nframes] "
-    "[-C min:max] [-c min:max] [-e enc]\n\t"
-    "[-f device] [-j flag] [-L addr] [-m mode] [-q port] [-r rate]\n\t"
-    "[-s name] [-t mode] [-U unit] [-v volume] [-w flag] [-z nframes]\n";
+    "[-C min:max] [-c min:max]\n\t"
+    "[-e enc] [-F device] [-f device] [-j flag] [-L addr] [-m mode]\n\t"
+    "[-Q port] [-q port] [-r rate] [-s name] [-t mode] [-U unit]\n\t"
+    "[-v volume] [-w flag] [-z nframes]\n";
+
+/*
+ * default audio devices
+ */
+static char *default_devs[] = {
+#ifdef DEFAULT_DEV
+	DEFAULT_DEV,
+#else
+	"rsnd/0", "rsnd/1", "rsnd/2", "rsnd/3",
+#endif
+	NULL
+};
+
+/*
+ * default MIDI ports
+ */
+static char *default_ports[] = {
+	"rmidi/0", "rmidi/1", "rmidi/2", "rmidi/3",
+	"rmidi/4", "rmidi/5", "rmidi/6", "rmidi/7",
+	NULL
+};
 
 /*
  * SIGINT handler, it raises the quit flag. If the flag is already set,
@@ -122,6 +138,16 @@ sigint(int s)
 	quit_flag = 1;
 }
 
+/*
+ * SIGHUP handler, it raises the reopen flag, which requests devices
+ * to be reopened.
+ */
+void
+sighup(int s)
+{
+	reopen_flag = 1;
+}
+
 void
 opt_ch(int *rcmin, int *rcmax)
 {
@@ -224,14 +250,16 @@ setsig(void)
 	struct sigaction sa;
 
 	quit_flag = 0;
+	reopen_flag = 0;
 	sigfillset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART;
 	sa.sa_handler = sigint;
-	if (sigaction(SIGINT, &sa, NULL) < 0)
+	if (sigaction(SIGINT, &sa, NULL) == -1)
 		err(1, "sigaction(int) failed");
-	if (sigaction(SIGTERM, &sa, NULL) < 0)
+	if (sigaction(SIGTERM, &sa, NULL) == -1)
 		err(1, "sigaction(term) failed");
-	if (sigaction(SIGHUP, &sa, NULL) < 0)
+	sa.sa_handler = sighup;
+	if (sigaction(SIGHUP, &sa, NULL) == -1)
 		err(1, "sigaction(hup) failed");
 }
 
@@ -243,11 +271,11 @@ unsetsig(void)
 	sigfillset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART;
 	sa.sa_handler = SIG_DFL;
-	if (sigaction(SIGHUP, &sa, NULL) < 0)
+	if (sigaction(SIGHUP, &sa, NULL) == -1)
 		err(1, "unsetsig(hup): sigaction failed");
-	if (sigaction(SIGTERM, &sa, NULL) < 0)
+	if (sigaction(SIGTERM, &sa, NULL) == -1)
 		err(1, "unsetsig(term): sigaction failed");
-	if (sigaction(SIGINT, &sa, NULL) < 0)
+	if (sigaction(SIGINT, &sa, NULL) == -1)
 		err(1, "unsetsig(int): sigaction failed");
 }
 
@@ -267,12 +295,12 @@ getbasepath(char *base)
 		snprintf(base, SOCKPATH_MAX, SOCKPATH_DIR "-%u", uid);
 	}
 	omask = umask(mask);
-	if (mkdir(base, 0777) < 0) {
+	if (mkdir(base, 0777) == -1) {
 		if (errno != EEXIST)
 			err(1, "mkdir(\"%s\")", base);
 	}
 	umask(omask);
-	if (stat(base, &sb) < 0)
+	if (stat(base, &sb) == -1)
 		err(1, "stat(\"%s\")", base);
 	if (!S_ISDIR(sb.st_mode))
 		errx(1, "%s is not a directory", base);
@@ -287,7 +315,8 @@ mkdev(char *path, struct aparams *par,
 	struct dev *d;
 
 	for (d = dev_list; d != NULL; d = d->next) {
-		if (strcmp(d->path, path) == 0)
+		if (d->alt_list->next == NULL &&
+		    strcmp(d->alt_list->name, path) == 0)
 			return d;
 	}
 	if (!bufsz && !round) {
@@ -309,7 +338,8 @@ mkport(char *path, int hold)
 	struct port *c;
 
 	for (c = port_list; c != NULL; c = c->next) {
-		if (strcmp(c->path, path) == 0)
+		if (c->path_list->next == NULL &&
+		    strcmp(c->path_list->str, path) == 0)
 			return c;
 	}
 	c = port_new(path, MODE_MIDIMASK, hold);
@@ -336,7 +366,7 @@ mkopt(char *path, struct dev *d,
 int
 main(int argc, char **argv)
 {
-	int c, background, unit;
+	int c, i, background, unit;
 	int pmin, pmax, rmin, rmax;
 	char base[SOCKPATH_MAX], path[SOCKPATH_MAX];
 	unsigned int mode, dup, mmc, vol;
@@ -357,11 +387,11 @@ main(int argc, char **argv)
 	/*
 	 * global options defaults
 	 */
-	vol = 118;
+	vol = 127;
 	dup = 1;
 	mmc = 0;
 	hold = 0;
-	autovol = 1;
+	autovol = 0;
 	bufsz = 0;
 	round = 0;
 	rate = DEFAULT_RATE;
@@ -374,8 +404,13 @@ main(int argc, char **argv)
 	aparams_init(&par);
 	mode = MODE_PLAY | MODE_REC;
 	tcpaddr_list = NULL;
+	d = NULL;
+	p = NULL;
 
-	while ((c = getopt(argc, argv, "a:b:c:C:de:f:j:L:m:q:r:s:t:U:v:w:x:z:")) != -1) {
+	slot_array_init();
+
+	while ((c = getopt(argc, argv,
+	    "a:b:c:C:de:F:f:j:L:m:Q:q:r:s:t:U:v:w:x:z:")) != -1) {
 		switch (c) {
 		case 'd':
 			log_level++;
@@ -421,16 +456,24 @@ main(int argc, char **argv)
 				errx(1, "%s: volume is %s", optarg, str);
 			break;
 		case 's':
-			if ((d = dev_list) == NULL) {
-				d = mkdev(DEFAULT_DEV, &par, 0, bufsz, round,
-				    rate, hold, autovol);
+			if (d == NULL) {
+				for (i = 0; default_devs[i] != NULL; i++) {
+					mkdev(default_devs[i], &par, 0,
+					    bufsz, round, rate, 0, autovol);
+				}
+				d = dev_list;
 			}
 			if (mkopt(optarg, d, pmin, pmax, rmin, rmax,
 				mode, vol, mmc, dup) == NULL)
 				return 1;
 			break;
 		case 'q':
-			mkport(optarg, hold);
+			p = mkport(optarg, hold);
+			break;
+		case 'Q':
+			if (p == NULL)
+				errx(1, "-Q %s: no ports defined", optarg);
+			namelist_add(&p->path_list, optarg);
 			break;
 		case 'a':
 			hold = opt_onoff();
@@ -449,9 +492,15 @@ main(int argc, char **argv)
 				errx(1, "%s: block size is %s", optarg, str);
 			break;
 		case 'f':
-			mkdev(optarg, &par, 0, bufsz, round,
+			d = mkdev(optarg, &par, 0, bufsz, round,
 			    rate, hold, autovol);
 			break;
+		case 'F':
+			if (d == NULL)
+				errx(1, "-F %s: no devices defined", optarg);
+			if (!dev_addname(d, optarg))
+				exit(1);
+			break;
 		default:
 			fputs(usagestr, stderr);
 			return 1;
@@ -463,8 +512,16 @@ main(int argc, char **argv)
 		fputs(usagestr, stderr);
 		return 1;
 	}
-	if (dev_list == NULL)
-		mkdev(DEFAULT_DEV, &par, 0, bufsz, round, rate, hold, autovol);
+	if (port_list == NULL) {
+		for (i = 0; default_ports[i] != NULL; i++)
+			mkport(default_ports[i], 0);
+	}
+	if (dev_list == NULL) {
+		for (i = 0; default_devs[i] != NULL; i++) {
+			mkdev(default_devs[i], &par, 0,
+			    bufsz, round, rate, 0, autovol);
+		}
+	}
 	for (d = dev_list; d != NULL; d = d->next) {
 		if (opt_byname(d, "default"))
 			continue;
@@ -505,20 +562,27 @@ main(int argc, char **argv)
 	if (background) {
 		log_flush();
 		log_level = 0;
-		if (daemon(0, 0) < 0)
+		if (daemon(0, 0) == -1)
 			err(1, "daemon");
 	}
 	if (pw != NULL) {
-		if (setpriority(PRIO_PROCESS, 0, SNDIO_PRIO) < 0)
+		if (setpriority(PRIO_PROCESS, 0, SNDIO_PRIO) == -1)
 			err(1, "setpriority");
-		if (setgroups(1, &pw->pw_gid) ||
-		    setgid(pw->pw_gid) ||
-		    setuid(pw->pw_uid))
+		if (setgroups(1, &pw->pw_gid) == -1 ||
+		    setgid(pw->pw_gid) == -1 ||
+		    setuid(pw->pw_uid) == -1)
 			err(1, "cannot drop privileges");
 	}
 	for (;;) {
 		if (quit_flag)
 			break;
+		if (reopen_flag) {
+			reopen_flag = 0;
+			for (d = dev_list; d != NULL; d = d->next)
+				dev_reopen(d);
+			for (p = port_list; p != NULL; p = p->next)
+				port_reopen(p);
+		}
 		if (!file_poll())
 			break;
 	}
@@ -534,6 +598,8 @@ main(int argc, char **argv)
 		; /* nothing */
 	midi_done();
 
+	while (opt_list)
+		opt_del(opt_list);
 	while (dev_list)
 		dev_del(dev_list);
 	while (port_list)
